@@ -35,16 +35,12 @@
  */
 
 #ifndef lint
-static char copyright[] =
+static const char *copyright =
 "@(#) Copyright (c) 1983, 1993\n\
 	The Regents of the University of California.  All rights reserved.\n";
-#endif /* not lint */
-
-#ifndef lint
-#if 0
-static char sccsid[] = "@(#)main.c	8.1 (Berkeley) 6/6/93";
-#endif
-static char rcsid[] = "$OpenBSD: main.c,v 1.4 1997/01/17 07:13:30 millert Exp $";
+/* static char sccsid[] = "@(#)main.c	8.1 (Berkeley) 6/6/93"; */
+/* static char rcsid[] = "$OpenBSD: main.c,v 1.4 1997/01/17 07:13:30 millert Exp $"; */
+static const char *rcsid = "tftp-hpa $Id: main.c,v 1.1.1.1 1999/09/26 06:32:42 hpa Exp $";
 #endif /* not lint */
 
 /* Many bug fixes are from Jim Guyton <guyton@rand-unix> */
