@@ -1,4 +1,6 @@
-/*	$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
+/* tftp-hpa: $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa Exp $ */
+
+/* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
 /*
  * Copyright (c) 1983 Regents of the University of California.
@@ -34,14 +36,12 @@
  */
 
 #ifndef lint
-char copyright[] =
+static const char *copyright =
 "@(#) Copyright (c) 1983 Regents of the University of California.\n\
  All rights reserved.\n";
-#endif /* not lint */
-
-#ifndef lint
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
-static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";
+/*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
+static const char *rcsid = "tftp-hpa $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -115,15 +115,17 @@ void timer(int);
 void justquit(int);
 void do_opt(char *, char *, char **);
 
-int     set_blksize(char *, char **);
-int     set_tsize(char *, char **);
-int	set_timeout(char *, char **);
+int set_blksize(char *, char **);
+int set_blksize2(char *, char **);
+int set_tsize(char *, char **);
+int set_timeout(char *, char **);
 
 struct options {
         char    *o_opt;
         int     (*o_fnc)(char *, char **);
 } options[] = {
         { "blksize",    set_blksize  },
+        { "blksize2",   set_blksize2  },
         { "tsize",      set_tsize },
 	{ "timeout",	set_timeout  },
         { NULL,         NULL }
@@ -396,6 +398,8 @@ tftp(struct tftphdr *tp, int size)
 	exit(1);
 }
 
+static int blksize_set;
+
 /*
  * Set a non-standard block size (c.f. RFC2348)
  */
@@ -403,7 +407,10 @@ int
 set_blksize(char *val, char **ret)
 {
   	static char b_ret[6];
-        int sz = atoi(val);
+        unsigned int sz = atoi(val);
+
+	if ( blksize_set )
+	  return 0;
 
         if (sz < 8)
                 return(0);
@@ -411,10 +418,46 @@ set_blksize(char *val, char **ret)
                 sz = MAX_SEGSIZE;
 
         segsize = sz;
-        sprintf(*ret = b_ret, "%d", sz);
+        sprintf(*ret = b_ret, "%u", sz);
+
+	blksize_set = 1;
+
         return(1);
 }
 
+/*
+ * Set a power-of-two block size (nonstandard)
+ */
+int
+set_blksize2(char *val, char **ret)
+{
+  	static char b_ret[6];
+        unsigned int sz = atoi(val);
+
+	if ( blksize_set )
+	  return 0;
+
+        if (sz < 8)
+                return(0);
+        else if (sz > MAX_SEGSIZE)
+	        sz = MAX_SEGSIZE;
+
+	/* Convert to a power of two */
+	if ( sz & (sz-1) ) {
+	  unsigned int sz1 = 1;
+	  /* Not a power of two - need to convert */
+	  while ( sz >>= 1 )
+	    sz1 <<= 1;
+	  sz = sz1;
+	}
+
+        segsize = sz;
+        sprintf(*ret = b_ret, "%u", sz);
+
+	blksize_set = 1;
+
+        return(1);
+}
 
 /*
  * Return a file size (c.f. RFC2349)
@@ -464,6 +507,9 @@ do_opt(char *opt, char *val, char **ap)
 {
      struct options *po;
      char *ret;
+
+     /* Global option-parsing variables initialization */
+     blksize_set = 0;
      
      if ( !*opt )
 	  return;
@@ -546,6 +592,7 @@ validate_access(char *filename, int mode, struct formats *pf)
 			if ((stbuf.st_mode&(S_IREAD >> 6)) == 0)
 				return (EACCESS);
 			tsize = stbuf.st_size;
+			/* We don't know the tsize if conversion is needed */
 			tsize_ok = !pf->f_convert;
 		} else {
 			if ((stbuf.st_mode&(S_IWRITE >> 6)) == 0)
